from __future__ import annotations

from django.contrib.auth import get_user_model
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import gettext_lazy as _

from .managers import GenericLinkQuerySet


class GenericLink(models.Model):
    """
    Relates an object with an url and its data
    """

    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField(db_index=True)
    content_object = GenericForeignKey("content_type", "object_id")

    url = models.URLField()
    title = models.CharField(max_length=200)
    description = models.TextField(max_length=1000, null=True, blank=True)

    user = models.ForeignKey(get_user_model(), null=True, blank=True, on_delete=models.SET_NULL)
    created_at = models.DateTimeField(auto_now_add=True, db_index=True)

    is_external = models.BooleanField(default=True, db_index=True)

    objects = GenericLinkQuerySet.as_manager()

    class Meta:
        ordering = ("-created_at",)
        verbose_name = _("Generic Link")
        verbose_name_plural = _("Generic Links")
        indexes = [
            models.Index(fields=["content_type", "object_id"]),
        ]

    def __str__(self):
        return f"{self.title} :: {self.url}"



from __future__ import annotations

from django import forms

from generic_links.models import GenericLink


class AddLinkForm(forms.ModelForm):
    class Meta:
        model = GenericLink
        fields = ("title", "url", "description", "is_external")

    def __init__(self, content_object, user, *args, **kwargs):
        self.content_object = content_object
        self.user = user
        super().__init__(*args, **kwargs)

    def save(self, *args, **kwargs):
        self.instance = GenericLink.objects.create(
            content_object=self.content_object,
            url=self.cleaned_data["url"].strip(),
            title=self.cleaned_data["title"],
            user=self.user,
            is_external=self.cleaned_data["is_external"],
        )
        self.instance.save()
        return self.instance



from __future__ import annotations

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "__first__"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="GenericLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("object_id", models.PositiveIntegerField(db_index=True)),
                ("url", models.URLField()),
                ("title", models.CharField(max_length=200)),
                (
                    "description",
                    models.TextField(max_length=1000, null=True, blank=True),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("is_external", models.BooleanField(default=True, db_index=True)),
                (
                    "content_type",
                    models.ForeignKey(to="contenttypes.ContentType", on_delete=models.CASCADE),
                ),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        to=settings.AUTH_USER_MODEL,
                        null=True,
                        on_delete=models.SET_NULL,
                    ),
                ),
            ],
            options={
                "ordering": ("-created_at",),
                "verbose_name": "Generic Link",
                "verbose_name_plural": "Generic Links",
            },
        ),
    ]



from __future__ import annotations

from generic_links.models import GenericLink


def get_links_for(obj, is_external=None):
    """
    Returns a queryset with all instances of GenericLink for a given object
    """

    params = {
        "object_id": obj.id,
        "content_type__app_label": obj._meta.app_label,
        "content_type__model": obj._meta.model_name,
    }

    if is_external is not None:
        params["is_external"] = is_external

    return GenericLink.objects.filter(**params).select_related("user")



from django.db import models


class GenericLinkQuerySet(models.QuerySet):
    def internal(self):
        return self.exclude(is_external=True)

    def external(self):
        return self.filter(is_external=True)

    def by_user(self, user):
        return self.filter(user=user)



# Generated by Django 5.0.1 on 2024-02-10 15:06

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("generic_links", "0001_initial"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AlterField(
            model_name="genericlink",
            name="id",
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID"),
        ),
        migrations.AddIndex(
            model_name="genericlink",
            index=models.Index(fields=["content_type", "object_id"], name="generic_lin_content_569ecc_idx"),
        ),
    ]



"""
Several useful template tags!
"""

from __future__ import annotations

from django import template
from django.db.models import Model, QuerySet

from generic_links import utils

register = template.Library()


@register.simple_tag
def get_links_for(obj: Model, is_external: bool | None = None) -> QuerySet:
    """
    Usage:

    {% get_links_for <obj> as <some_var> %}

    or

    {% get_links_for <obj> <boolean> as <some_var> %}
    """

    return utils.get_links_for(obj, is_external=is_external)



from __future__ import annotations

from django.contrib import admin
from django.contrib.contenttypes import admin as ct_admin

from generic_links.models import GenericLink


class GenericLinkStackedInline(ct_admin.GenericStackedInline):
    model = GenericLink
    extra = 1


class GenericLinkTabularInline(ct_admin.GenericTabularInline):
    model = GenericLink
    extra = 1


@admin.register(GenericLink)
class GenericLinkAdmin(admin.ModelAdmin):
    list_display = ("title", "url", "description", "created_at", "user", "is_external")
    search_fields = ("title", "url", "user__username")
    list_filter = (
        "is_external",
        "created_at",
    )
    raw_id_fields = ("user",)
